@@ -183,7 +183,7 @@ gdm_auth_secure_display (GdmDisplay *d)
 	    int authfd;
 
 	    /* Note, Xnest can't use the GDM_KEY_SERV_AUTHDIR unless running as
-	     * root, which is rare anyway, unless the user is a wanker */
+	     * root, which is rare anyway. */
 
 	    d->authfile = g_build_filename (gdm_get_value_string (GDM_KEY_USER_AUTHDIR_FALLBACK), ".gdmXXXXXX", NULL);
 
@@ -710,7 +710,9 @@ try_user_add_again:
 	af = gdm_safe_fopen_ap (d->userauth);
     }
 
+    /* Set to NULL, because can goto try_user_add_again. */
     g_free (authdir);
+    authdir = NULL;
 
     if G_UNLIKELY (af == NULL) {
 	/* Really no need to clean up here - this process is a goner anyway */
@@ -982,7 +984,7 @@ gdm_auth_purge (GdmDisplay *d, FILE *af, gboolean remove_when_empty)
 		keep = g_slist_append (keep, xa);
 
 	/* just being ultra anal */
-	cnt ++;
+	cnt++;
 	if (cnt > 500)
 		break;
     }
