@@ -184,6 +184,7 @@ static gboolean GdmAllowRoot = FALSE;
 static gboolean GdmAllowRemoteRoot = FALSE;
 static gboolean GdmAllowRemoteAutoLogin = FALSE;
 static gboolean GdmCheckDirOwner = TRUE;
+static gboolean GdmSupportAutomount = FALSE;
 static gboolean GdmTimedLoginEnable = FALSE;
 static gboolean GdmDynamicXservers = FALSE;
 static gboolean GdmVTAllocation = TRUE;
@@ -374,6 +375,7 @@ gdm_config_init (void)
    gdm_config_add_hash (GDM_KEY_CONSOLE_NOTIFY, &GdmConsoleNotify, &bool_type);
    gdm_config_add_hash (GDM_KEY_TIMED_LOGIN_ENABLE, &GdmTimedLoginEnable, &bool_type);
    gdm_config_add_hash (GDM_KEY_CHECK_DIR_OWNER, &GdmCheckDirOwner, &bool_type);
+   gdm_config_add_hash (GDM_KEY_SUPPORT_AUTOMOUNT, &GdmSupportAutomount, &bool_type);
    gdm_config_add_hash (GDM_KEY_XDMCP, &GdmXdmcp, &bool_type);
    gdm_config_add_hash (GDM_KEY_INDIRECT, &GdmIndirect, &bool_type);
    gdm_config_add_hash (GDM_KEY_XDMCP_PROXY, &GdmXdmcpProxy, &bool_type);
@@ -816,10 +818,9 @@ gdm_config_key_to_string_per_display (gchar *display, gchar *key, gchar **retval
 
    file = gdm_get_per_display_custom_config_file (display);
 
-   if (splitstr != NULL &&
-       (strcmp (ve_sure_string (splitstr[0]), "greeter") == 0 ||
-        strcmp (ve_sure_string (splitstr[0]), "gui") == 0 ||
-        is_key (key, GDM_KEY_PAM_STACK))) {
+   if (strcmp (ve_sure_string (splitstr[0]), "greeter") == 0 ||
+       strcmp (ve_sure_string (splitstr[0]), "gui") == 0 ||
+       is_key (key, GDM_KEY_PAM_STACK)) {
       gdm_config_key_to_string (file, key, retval);
    }
 
@@ -841,17 +842,18 @@ gdm_config_key_to_string (gchar *file, gchar *key, gchar **retval)
    VeConfig *cfg = ve_config_get (file);
    GdmConfigType *type = gdm_config_hash_lookup (type_hash, key);
    gchar **splitstr = g_strsplit (key, "/", 2);
+   GList *list;
    *retval = NULL;
 
    /* Should not fail, all keys should have a category. */
-   if (splitstr == NULL || splitstr[0] == NULL)
+   if (splitstr[0] == NULL)
       return;
 
    /* If file doesn't exist, then just return */
    if (cfg == NULL)
       return;
 
-   GList *list = ve_config_get_keys (cfg, splitstr[0]);
+   list = ve_config_get_keys (cfg, splitstr[0]);
    while (list != NULL) {
       gchar *display_key     = (char *)list->data;
       gchar *display_fullkey = g_strdup_printf ("%s/%s", splitstr[0], display_key);
@@ -1705,7 +1707,7 @@ gdm_update_config (gchar* key)
    if (custom_cfg != NULL) {
        gchar **splitstr = g_strsplit (key, "/", 2);
 
-       if (splitstr != NULL && splitstr[0] != NULL) {
+       if (splitstr[0] != NULL) {
           GList *list = ve_config_get_keys (custom_cfg, splitstr[0]);
 
           while (list != NULL) {
@@ -1893,7 +1895,7 @@ gdm_load_config_option (gpointer key_in, gpointer value_in, gpointer data)
       /* First check the custom file */
       if (cfgfiles->custom_cfg != NULL) {
           gchar **splitstr = g_strsplit (key_in, "/", 2);
-          if (splitstr != NULL && splitstr[0] != NULL) {
+          if (splitstr[0] != NULL) {
              GList *list = ve_config_get_keys (cfgfiles->custom_cfg, splitstr[0]);
 
              while (list != NULL) {
